@@ -25,10 +25,13 @@
  */
 
 #include <config.h>
+
 #include <sys/time.h>
 #include <unistd.h>
-#include <gal/util/e-util.h>
-#include <gal/util/e-i18n.h>
+
+#include "gal/util/e-i18n.h"
+#include "gal/util/e-util.h"
+
 #include "e-cell-float.h"
 
 #define PARENT_TYPE e_cell_text_get_type ()
