@@ -22,9 +22,12 @@
  */
 
 #include <config.h>
+
 #include <sys/time.h>
 #include <unistd.h>
-#include <gal/util/e-util.h>
+
+#include "gal/util/e-util.h"
+
 #include "e-cell-size.h"
 
 #define PARENT_TYPE e_cell_text_get_type ()
@@ -61,7 +64,7 @@ e_cell_size_class_init (GtkObjectClass *object_class)
 {
 	ECellTextClass *ectc = (ECellTextClass *) object_class;
 
-	parent_class = gtk_type_class (PARENT_TYPE);
+	parent_class = g_type_class_ref (PARENT_TYPE);
 
 	ectc->get_text  = ecd_get_text;
 	ectc->free_text = ecd_free_text;
@@ -82,23 +85,25 @@ e_cell_size_init (GtkObject *object)
  * is interpreted as being a time_t.
  *
  * The ECellSize object support a large set of properties that can be
- * configured through the Gtk argument system and allows the user to have
- * a finer control of the way the string is displayed.  The arguments supported
- * allow the control of strikeout, bold, color and a size filter.
+ * configured through the Gtk argument system and allows the user to
+ * have a finer control of the way the string is displayed.  The
+ * arguments supported allow the control of strikeout, underline,
+ * bold, color and a size filter.
  *
- * The arguments "strikeout_column", "bold_column" and "color_column" set
- * and return an integer that points to a column in the model that controls
- * these settings.  So controlling the way things are rendered is achieved
- * by having special columns in the model that will be used to flag whether
- * the size should be rendered with strikeout, or bolded.   In the case of
- * the "color_column" argument, the column in the model is expected to have
- * a string that can be parsed by gdk_color_parse().
+ * The arguments "strikeout_column", "underline_column", "bold_column"
+ * and "color_column" set and return an integer that points to a
+ * column in the model that controls these settings.  So controlling
+ * the way things are rendered is achieved by having special columns
+ * in the model that will be used to flag whether the size should be
+ * rendered with strikeout, underline, or bolded.  In the case of the
+ * "color_column" argument, the column in the model is expected to
+ * have a string that can be parsed by gdk_color_parse().
  * 
  * Returns: an ECell object that can be used to render file sizes.  */
 ECell *
 e_cell_size_new (const char *fontname, GtkJustification justify)
 {
-	ECellSize *ecd = gtk_type_new (e_cell_size_get_type ());
+	ECellSize *ecd = g_object_new (E_CELL_SIZE_TYPE, NULL);
 
 	e_cell_text_construct(E_CELL_TEXT(ecd), fontname, justify);
       
