@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * e-table-utils.c
  * Copyright 2000, 2001, Ximian, Inc.
  *
@@ -22,9 +22,12 @@
  */
 
 #include <config.h>
-#include "gal/util/e-i18n.h"
+
+#include <libintl.h>		/* This file uses dgettext() but no _() */
+
 #include "gal/util/e-util.h"
 #include "gal/widgets/e-unicode.h"
+
 #include "e-table-utils.h"
 #include "e-table-header-utils.h"
 
@@ -34,16 +37,17 @@ e_table_state_to_header (GtkWidget *widget, ETableHeader *full_header, ETableSta
 	ETableHeader *nh;
 	const int max_cols = e_table_header_count (full_header);
 	int column;
+	GValue *val = g_new0 (GValue, 1);
 
 	g_return_val_if_fail (widget, NULL);
 	g_return_val_if_fail (full_header, NULL);
 	g_return_val_if_fail (state, NULL);
 
 	nh = e_table_header_new ();
-
-	gtk_object_set(GTK_OBJECT(nh),
-		       "width_extras", e_table_header_width_extras(widget->style),
-		       NULL);
+	g_value_init (val, G_TYPE_DOUBLE);
+	g_value_set_double (val, e_table_header_width_extras (widget->style));
+	g_object_set_property (G_OBJECT(nh), "width_extras", val);
+	g_free (val);
 
 	for (column = 0; column < state->col_count; column++) {
 		int col;
@@ -87,7 +91,7 @@ et_col_spec_to_col (ETableColumnSpecification *col_spec,
 	if (cell && compare) {
 		char *title = dgettext (domain, col_spec->title);
 
-		title = e_utf8_from_locale_string (title);
+		title = g_strdup (title);
 
 		if (col_spec->pixbuf && *col_spec->pixbuf) {
 			GdkPixbuf *pixbuf;
@@ -113,9 +117,9 @@ et_col_spec_to_col (ETableColumnSpecification *col_spec,
 		g_free (title);
 	}
 	if (col && col_spec->compare_col != col_spec->model_col)
-		gtk_object_set (GTK_OBJECT (col),
-				"compare_col", col_spec->compare_col,
-				NULL);
+		g_object_set (col,
+			      "compare_col", col_spec->compare_col,
+			      NULL);
 	return col;
 }
 
@@ -135,8 +139,10 @@ e_table_spec_to_full_header (ETableSpecification *spec,
 		ETableCol *col = et_col_spec_to_col (
 			spec->columns[column], ete, spec->domain);
 
-		if (col)
+		if (col) {
 			e_table_header_add_column (nh, col, -1);
+			g_object_unref (col);
+		}
 	}
 
 	return nh;
