@@ -33,23 +33,30 @@
 #include <libxml/parser.h>
 #include <libxml/xmlmemory.h>
 
-#include <gtk/gtksignal.h>
 #include "gal/util/e-util.h"
 #include "gal/util/e-xml-utils.h"
-#include "e-tree-memory.h"
 
-#define PARENT_TYPE E_TREE_MODEL_TYPE
+#include "e-tree-memory.h"
 
 #define TREEPATH_CHUNK_AREA_SIZE (30 * sizeof (ETreeMemoryPath))
 
-static ETreeModel *parent_class;
+static ETreeModelClass *parent_class;
 static GMemChunk  *node_chunk;
 
+enum {
+	FILL_IN_CHILDREN,
+	LAST_SIGNAL
+};
+
+static guint signals [LAST_SIGNAL] = { 0, };
+
 typedef struct ETreeMemoryPath ETreeMemoryPath;
 
 struct ETreeMemoryPath {
 	gpointer         node_data;
 
+	guint            children_computed : 1;
+
 	/* parent/child/sibling pointers */
 	ETreeMemoryPath *parent;
 	ETreeMemoryPath *next_sibling;
@@ -71,6 +78,16 @@ struct ETreeMemoryPriv {
 
 /* ETreeMemoryPath functions */
 
+static inline void
+check_children (ETreeMemory *memory, ETreePath node)
+{
+	ETreeMemoryPath *path = node;
+	if (!path->children_computed) {
+		g_signal_emit (G_OBJECT (memory), signals[FILL_IN_CHILDREN], 0, node);
+		path->children_computed = TRUE;
+	}
+}
+
 static int
 e_tree_memory_path_depth (ETreeMemoryPath *path)
 {
@@ -164,7 +181,7 @@ e_tree_memory_freeze(ETreeMemory *etmm)
 	ETreeMemoryPriv *priv = etmm->priv;
 
 	if (priv->frozen == 0)
-	e_tree_model_pre_change(E_TREE_MODEL(etmm));
+		e_tree_model_pre_change(E_TREE_MODEL(etmm));
 
 	priv->frozen ++;
 }
@@ -194,7 +211,7 @@ e_tree_memory_thaw(ETreeMemory *etmm)
 /* virtual methods */
 
 static void
-etmm_destroy (GtkObject *object)
+etmm_dispose (GObject *object)
 {
 	ETreeMemory *etmm = E_TREE_MEMORY (object);
 	ETreeMemoryPriv *priv = etmm->priv;
@@ -209,7 +226,7 @@ etmm_destroy (GtkObject *object)
 	}
 	etmm->priv = NULL;
 
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static ETreePath
@@ -230,6 +247,8 @@ static ETreePath
 etmm_get_first_child (ETreeModel *etm, ETreePath node)
 {
 	ETreeMemoryPath *path = node;
+
+	check_children (E_TREE_MEMORY (etm), node);
 	return path->first_child;
 }
 
@@ -237,6 +256,8 @@ static ETreePath
 etmm_get_last_child (ETreeModel *etm, ETreePath node)
 {
 	ETreeMemoryPath *path = node;
+
+	check_children (E_TREE_MEMORY (etm), node);
 	return path->last_child;
 }
 
@@ -265,6 +286,8 @@ static gboolean
 etmm_is_expandable (ETreeModel *etm, ETreePath node)
 {
 	ETreeMemoryPath *path = node;
+
+	check_children (E_TREE_MEMORY (etm), node);
 	return path->first_child != NULL;
 }
 
@@ -274,13 +297,15 @@ etmm_get_children (ETreeModel *etm, ETreePath node, ETreePath **nodes)
 	ETreeMemoryPath *path = node;
 	guint n_children;
 
+	check_children (E_TREE_MEMORY (etm), node);
+
 	n_children = path->num_children;
 
 	if (nodes) {
 		ETreeMemoryPath *p;
 		int i = 0;
 
-		(*nodes) = g_malloc (sizeof (ETreePath) * n_children);
+		(*nodes) = g_new (ETreePath, n_children);
 		for (p = path->first_child; p; p = p->next_sibling) {
 			(*nodes)[i++] = p;
 		}
@@ -304,34 +329,68 @@ etmm_get_expanded_default (ETreeModel *etm)
 	return priv->expanded_default;
 }
 
-
 static void
-e_tree_memory_class_init (GtkObjectClass *klass)
+etmm_clear_children_computed (ETreeMemoryPath *path)
 {
-	ETreeModelClass *tree_class = (ETreeModelClass *) klass;
-
-	parent_class                     = gtk_type_class (PARENT_TYPE);
-	
-	node_chunk                       = g_mem_chunk_create (ETreeMemoryPath, TREEPATH_CHUNK_AREA_SIZE, G_ALLOC_AND_FREE);
-
-	klass->destroy                   = etmm_destroy;
-
-	tree_class->get_root             = etmm_get_root;
-	tree_class->get_prev             = etmm_get_prev;
-	tree_class->get_next             = etmm_get_next;
-	tree_class->get_first_child      = etmm_get_first_child;
-	tree_class->get_last_child       = etmm_get_last_child;
-	tree_class->get_parent           = etmm_get_parent;
-
-	tree_class->is_root              = etmm_is_root;
-	tree_class->is_expandable        = etmm_is_expandable;
-	tree_class->get_children         = etmm_get_children;
-	tree_class->depth                = etmm_depth;
-	tree_class->get_expanded_default = etmm_get_expanded_default;
+	for (path = path->first_child; path; path = path->next_sibling) {
+		path->children_computed = FALSE;
+		etmm_clear_children_computed (path);
+	}
 }
 
 static void
-e_tree_memory_init (GtkObject *object)
+etmm_node_request_collapse (ETreeModel *etm, ETreePath node)
+{
+	if (node)
+		etmm_clear_children_computed (node);
+
+	if (parent_class->node_request_collapse) {
+		parent_class->node_request_collapse (etm, node);
+	}
+}
+
+
+static void
+e_tree_memory_class_init (ETreeMemoryClass *klass)
+{
+	ETreeModelClass *tree_class = (ETreeModelClass *) klass;
+	GObjectClass  *object_class = (GObjectClass *) klass;
+
+	parent_class                     = g_type_class_peek_parent (klass);
+	
+	node_chunk                       = g_mem_chunk_create (ETreeMemoryPath, TREEPATH_CHUNK_AREA_SIZE, G_ALLOC_AND_FREE);
+
+	signals [FILL_IN_CHILDREN] =
+		g_signal_new ("fill_in_children",
+			      E_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ETreeMemoryClass, fill_in_children),
+			      (GSignalAccumulator) NULL, NULL,
+			      g_cclosure_marshal_VOID__POINTER,
+			      G_TYPE_NONE, 1, G_TYPE_POINTER);
+
+	object_class->dispose             = etmm_dispose;
+
+	tree_class->get_root              = etmm_get_root;
+	tree_class->get_prev              = etmm_get_prev;
+	tree_class->get_next              = etmm_get_next;
+	tree_class->get_first_child       = etmm_get_first_child;
+	tree_class->get_last_child        = etmm_get_last_child;
+	tree_class->get_parent            = etmm_get_parent;
+
+	tree_class->is_root               = etmm_is_root;
+	tree_class->is_expandable         = etmm_is_expandable;
+	tree_class->get_children          = etmm_get_children;
+	tree_class->depth                 = etmm_depth;
+	tree_class->get_expanded_default  = etmm_get_expanded_default;
+
+	tree_class->node_request_collapse = etmm_node_request_collapse;
+
+	klass->fill_in_children           = NULL;
+}
+
+static void
+e_tree_memory_init (GObject *object)
 {
 	ETreeMemory *etmm = (ETreeMemory *)object;
 
@@ -347,7 +406,7 @@ e_tree_memory_init (GtkObject *object)
 	priv->destroy_user_data = NULL;
 }
 
-E_MAKE_TYPE(e_tree_memory, "ETreeMemory", ETreeMemory, e_tree_memory_class_init, e_tree_memory_init, PARENT_TYPE)
+E_MAKE_TYPE(e_tree_memory, "ETreeMemory", ETreeMemory, e_tree_memory_class_init, e_tree_memory_init, E_TREE_MODEL_TYPE)
 
 
 
@@ -372,13 +431,7 @@ e_tree_memory_construct (ETreeMemory *etmm)
 ETreeMemory *
 e_tree_memory_new (void)
 {
-	ETreeMemory *etmm;
-
-	etmm = gtk_type_new (e_tree_memory_get_type ());
-
-	e_tree_memory_construct(etmm);
-
-	return etmm;
+	return (ETreeMemory *) g_object_new (E_TREE_MEMORY_TYPE, NULL);
 }
 
 void
@@ -459,6 +512,7 @@ e_tree_memory_node_insert (ETreeMemory *tree_model,
 	new_path = g_chunk_new0 (ETreeMemoryPath, node_chunk);
 
 	new_path->node_data = node_data;
+	new_path->children_computed = FALSE;
 
 	if (parent_path != NULL) {
 		e_tree_memory_path_insert (parent_path, position, new_path);
