@@ -28,8 +28,11 @@
  */
 
 #include <config.h>
-#include <gtk/gtksignal.h>
-#include <gal/util/e-util.h>
+
+#include <gtk/gtk.h>
+
+#include "gal/util/e-util.h"
+
 #include "e-shortcut-model.h"
 
 /* This contains information on one item. */
