@@ -27,9 +27,11 @@
  */
 
 #include <config.h>
-#include <gtk/gtkarrow.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtksignal.h>
+
+#include <gtk/gtk.h>
+
+#include "gal/util/e-util.h"
+
 #include "e-vscrolled-bar.h"
 
 /* These are the offsets of the up & down buttons from the right and top/bottom
@@ -42,16 +44,16 @@
 
 static void e_vscrolled_bar_class_init (EVScrolledBarClass *class);
 static void e_vscrolled_bar_init (EVScrolledBar *vscrolled_bar);
-static void e_vscrolled_bar_destroy (GtkObject *object);
-static void e_vscrolled_bar_finalize (GtkObject *object);
+static void e_vscrolled_bar_dispose (GObject *object);
+static void e_vscrolled_bar_finalize (GObject *object);
 static void e_vscrolled_bar_map (GtkWidget *widget);
 static void e_vscrolled_bar_unmap (GtkWidget *widget);
 static void e_vscrolled_bar_size_request (GtkWidget      *widget,
 					  GtkRequisition *requisition);
 static void e_vscrolled_bar_size_allocate (GtkWidget *widget,
 					   GtkAllocation *allocation);
-static void e_vscrolled_bar_draw (GtkWidget    *widget,
-				  GdkRectangle *area);
+static int  e_vscrolled_bar_expose (GtkWidget      *widget,
+				    GdkEventExpose *event);
 static gint e_vscrolled_bar_button_press   (GtkWidget        *widget,
 					    GdkEventButton   *event);
 static void e_vscrolled_bar_add (GtkContainer *container,
@@ -73,57 +75,40 @@ static void e_vscrolled_bar_button_clicked (GtkWidget *button,
 static gboolean e_vscrolled_bar_scroll (gpointer data);
 static gboolean e_vscrolled_bar_timeout_handler (gpointer data);
 
-
+#define PARENT_TYPE GTK_TYPE_BIN
 static GtkBinClass *parent_class;
 
 
-GtkType
-e_vscrolled_bar_get_type (void)
-{
-	static GtkType e_vscrolled_bar_type = 0;
-
-	if (!e_vscrolled_bar_type) {
-		GtkTypeInfo e_vscrolled_bar_info = {
-			"EVScrolledBar",
-			sizeof (EVScrolledBar),
-			sizeof (EVScrolledBarClass),
-			(GtkClassInitFunc) e_vscrolled_bar_class_init,
-			(GtkObjectInitFunc) e_vscrolled_bar_init,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		parent_class = gtk_type_class (GTK_TYPE_BIN);
-		e_vscrolled_bar_type = gtk_type_unique (GTK_TYPE_BIN,
-							&e_vscrolled_bar_info);
-	}
-
-	return e_vscrolled_bar_type;
-}
-
+E_MAKE_TYPE (e_vscrolled_bar,
+	     "EVScrolledBar",
+	     EVScrolledBar,
+	     e_vscrolled_bar_class_init,
+	     e_vscrolled_bar_init,
+	     PARENT_TYPE)
 
 static void
 e_vscrolled_bar_class_init (EVScrolledBarClass *class)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	GtkWidgetClass *widget_class;
 	GtkContainerClass *container_class;
 
-	object_class = (GtkObjectClass *) class;
+	parent_class = g_type_class_ref (PARENT_TYPE);
+
+	object_class = (GObjectClass *) class;
 	widget_class = (GtkWidgetClass *) class;
 	container_class = (GtkContainerClass *) class;
 
 	/* Method override */
-	object_class->destroy		= e_vscrolled_bar_destroy;
-	object_class->finalize		= e_vscrolled_bar_finalize;
+	object_class->dispose		= e_vscrolled_bar_dispose;
+	object_class->finalize          = e_vscrolled_bar_finalize;
 
 	widget_class->map		= e_vscrolled_bar_map;
 	widget_class->unmap		= e_vscrolled_bar_unmap;
 	widget_class->size_request	= e_vscrolled_bar_size_request;
  	widget_class->size_allocate	= e_vscrolled_bar_size_allocate;
-	widget_class->draw		= e_vscrolled_bar_draw;
-	widget_class->button_press_event   = e_vscrolled_bar_button_press;
+	widget_class->button_press_event= e_vscrolled_bar_button_press;
+	widget_class->expose_event      = e_vscrolled_bar_expose;
 
 	container_class->add		= e_vscrolled_bar_add;
 	container_class->remove		= e_vscrolled_bar_remove;
@@ -141,35 +126,46 @@ e_vscrolled_bar_init (EVScrolledBar *vscrolled_bar)
 	gtk_container_set_resize_mode (GTK_CONTAINER (vscrolled_bar),
 				       GTK_RESIZE_QUEUE);
 
+	gtk_widget_set_redraw_on_allocate (GTK_WIDGET (vscrolled_bar), TRUE);
+
 	gtk_widget_push_composite_child ();
 
+	vscrolled_bar->up_ebox = gtk_event_box_new ();
 	vscrolled_bar->up_button = gtk_button_new ();
 	gtk_widget_set_composite_name (vscrolled_bar->up_button,
 				       "up_button");
-	gtk_widget_set_parent (vscrolled_bar->up_button,
-			       GTK_WIDGET (vscrolled_bar));
-	arrow = gtk_arrow_new (GTK_ARROW_UP, GTK_SHADOW_OUT);
-	gtk_misc_set_padding (GTK_MISC (arrow), 1, 1);
+	gtk_widget_set_parent (vscrolled_bar->up_ebox, GTK_WIDGET (vscrolled_bar));
+	arrow = gtk_arrow_new (GTK_ARROW_UP, GTK_SHADOW_NONE);
 	gtk_widget_show (arrow);
 	gtk_container_add (GTK_CONTAINER (vscrolled_bar->up_button), arrow);
 	gtk_widget_show (vscrolled_bar->up_button);
-	gtk_signal_connect_after (GTK_OBJECT (vscrolled_bar->up_button), "pressed", GTK_SIGNAL_FUNC (e_vscrolled_bar_button_pressed), vscrolled_bar);
-	gtk_signal_connect_after (GTK_OBJECT (vscrolled_bar->up_button), "released", GTK_SIGNAL_FUNC (e_vscrolled_bar_button_released), vscrolled_bar);
-	gtk_signal_connect (GTK_OBJECT (vscrolled_bar->up_button), "clicked", GTK_SIGNAL_FUNC (e_vscrolled_bar_button_clicked), vscrolled_bar);
+	gtk_container_add (GTK_CONTAINER (vscrolled_bar->up_ebox), vscrolled_bar->up_button);
+	gtk_widget_show (vscrolled_bar->up_ebox);
+	g_signal_connect_after (vscrolled_bar->up_button, "pressed",
+				G_CALLBACK (e_vscrolled_bar_button_pressed), vscrolled_bar);
+	g_signal_connect_after (vscrolled_bar->up_button, "released",
+				G_CALLBACK (e_vscrolled_bar_button_released), vscrolled_bar);
+	g_signal_connect (vscrolled_bar->up_button, "clicked",
+			  G_CALLBACK (e_vscrolled_bar_button_clicked), vscrolled_bar);
 
+	vscrolled_bar->down_ebox = gtk_event_box_new ();
 	vscrolled_bar->down_button = gtk_button_new ();
 	gtk_widget_set_composite_name (vscrolled_bar->up_button,
 				       "down_button");
-	gtk_widget_set_parent (vscrolled_bar->down_button,
+	gtk_widget_set_parent (vscrolled_bar->down_ebox,
 			       GTK_WIDGET (vscrolled_bar));
-	arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT);
-	gtk_misc_set_padding (GTK_MISC (arrow), 1, 1);
+	arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_NONE);
 	gtk_widget_show (arrow);
 	gtk_container_add (GTK_CONTAINER (vscrolled_bar->down_button), arrow);
 	gtk_widget_show (vscrolled_bar->down_button);
-	gtk_signal_connect_after (GTK_OBJECT (vscrolled_bar->down_button), "pressed", GTK_SIGNAL_FUNC (e_vscrolled_bar_button_pressed), vscrolled_bar);
-	gtk_signal_connect_after (GTK_OBJECT (vscrolled_bar->down_button), "released", GTK_SIGNAL_FUNC (e_vscrolled_bar_button_released), vscrolled_bar);
-	gtk_signal_connect (GTK_OBJECT (vscrolled_bar->down_button), "clicked", GTK_SIGNAL_FUNC (e_vscrolled_bar_button_clicked), vscrolled_bar);
+	gtk_container_add (GTK_CONTAINER (vscrolled_bar->down_ebox), vscrolled_bar->down_button);
+	gtk_widget_show (vscrolled_bar->down_ebox);
+	g_signal_connect_after (vscrolled_bar->down_button, "pressed",
+				G_CALLBACK (e_vscrolled_bar_button_pressed), vscrolled_bar);
+	g_signal_connect_after (vscrolled_bar->down_button, "released",
+				G_CALLBACK (e_vscrolled_bar_button_released), vscrolled_bar);
+	g_signal_connect (vscrolled_bar->down_button, "clicked",
+			  G_CALLBACK (e_vscrolled_bar_button_clicked), vscrolled_bar);
 
 	gtk_widget_pop_composite_child ();
 
@@ -194,7 +190,7 @@ e_vscrolled_bar_new (GtkAdjustment *adjustment)
 {
 	GtkWidget *vscrolled_bar;
 
-	vscrolled_bar = GTK_WIDGET (gtk_type_new (e_vscrolled_bar_get_type ()));
+	vscrolled_bar = GTK_WIDGET (g_object_new (E_VSCROLLED_BAR_TYPE, NULL));
 	e_vscrolled_bar_set_adjustment (E_VSCROLLED_BAR (vscrolled_bar),
 					adjustment);
 
@@ -203,7 +199,7 @@ e_vscrolled_bar_new (GtkAdjustment *adjustment)
 
 
 static void
-e_vscrolled_bar_destroy (GtkObject *object)
+e_vscrolled_bar_dispose (GObject *object)
 {
 	EVScrolledBar *vscrolled_bar;
 
@@ -214,23 +210,28 @@ e_vscrolled_bar_destroy (GtkObject *object)
 		vscrolled_bar->timeout_id = 0;
 	}
 
-	gtk_widget_unparent (vscrolled_bar->up_button);
-	gtk_widget_unparent (vscrolled_bar->down_button);
+	if (vscrolled_bar->up_ebox)
+		gtk_widget_unparent (vscrolled_bar->up_ebox);
+	vscrolled_bar->up_ebox = NULL;
 
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	if (vscrolled_bar->down_ebox)
+		gtk_widget_unparent (vscrolled_bar->down_ebox);
+	vscrolled_bar->down_ebox = NULL;
+
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 
 static void
-e_vscrolled_bar_finalize (GtkObject *object)
+e_vscrolled_bar_finalize (GObject *object)
 {
 	EVScrolledBar *vscrolled_bar;
 
 	vscrolled_bar = E_VSCROLLED_BAR (object);
 
-	gtk_object_unref (GTK_OBJECT (vscrolled_bar->adjustment));
+	g_object_unref (vscrolled_bar->adjustment);
 
-	GTK_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 
@@ -240,7 +241,6 @@ e_vscrolled_bar_map (GtkWidget *widget)
 	EVScrolledBar *vscrolled_bar;
 	GtkAdjustment *adjustment;
 
-	g_return_if_fail (widget != NULL);
 	g_return_if_fail (E_IS_VSCROLLED_BAR (widget));
 
 	vscrolled_bar = E_VSCROLLED_BAR (widget);
@@ -250,15 +250,15 @@ e_vscrolled_bar_map (GtkWidget *widget)
   
 	adjustment = vscrolled_bar->adjustment;
 
-	if (GTK_WIDGET_VISIBLE (vscrolled_bar->up_button)
+	if (GTK_WIDGET_VISIBLE (vscrolled_bar->up_ebox)
 	    && adjustment->value != adjustment->lower
-	    && !GTK_WIDGET_MAPPED (vscrolled_bar->up_button))
-		gtk_widget_map (vscrolled_bar->up_button);
-  
-	if (GTK_WIDGET_VISIBLE (vscrolled_bar->down_button)
+	    && !GTK_WIDGET_MAPPED (vscrolled_bar->up_ebox))
+		gtk_widget_map (vscrolled_bar->up_ebox);
+	
+	if (GTK_WIDGET_VISIBLE (vscrolled_bar->down_ebox)
 	    && adjustment->value < adjustment->upper - adjustment->page_size
-	    && !GTK_WIDGET_MAPPED (vscrolled_bar->down_button))
-		gtk_widget_map (vscrolled_bar->down_button);
+	    && !GTK_WIDGET_MAPPED (vscrolled_bar->down_ebox))
+		gtk_widget_map (vscrolled_bar->down_ebox);
 }
 
 
@@ -267,7 +267,6 @@ e_vscrolled_bar_unmap (GtkWidget *widget)
 {
 	EVScrolledBar *vscrolled_bar;
 
-	g_return_if_fail (widget != NULL);
 	g_return_if_fail (E_IS_VSCROLLED_BAR (widget));
 
 	vscrolled_bar = E_VSCROLLED_BAR (widget);
@@ -275,11 +274,11 @@ e_vscrolled_bar_unmap (GtkWidget *widget)
 	/* chain parent class handler to unmap self and child */
 	GTK_WIDGET_CLASS (parent_class)->unmap (widget);
   
-	if (GTK_WIDGET_MAPPED (vscrolled_bar->up_button))
-		gtk_widget_unmap (vscrolled_bar->up_button);
+	if (GTK_WIDGET_MAPPED (vscrolled_bar->up_ebox))
+		gtk_widget_unmap (vscrolled_bar->up_ebox);
   
-	if (GTK_WIDGET_MAPPED (vscrolled_bar->down_button))
-		gtk_widget_unmap (vscrolled_bar->down_button);
+	if (GTK_WIDGET_MAPPED (vscrolled_bar->down_ebox))
+		gtk_widget_unmap (vscrolled_bar->down_ebox);
 }
 
 
@@ -291,7 +290,6 @@ e_vscrolled_bar_size_request (GtkWidget      *widget,
 	GtkBin *bin;
 	GtkRequisition child_requisition;
 
-	g_return_if_fail (widget != NULL);
 	g_return_if_fail (E_IS_VSCROLLED_BAR (widget));
 	g_return_if_fail (requisition != NULL);
 
@@ -309,12 +307,12 @@ e_vscrolled_bar_size_request (GtkWidget      *widget,
 	}
 
 	/* We remember the requested heights of the up & down buttons. */
-	gtk_widget_size_request (vscrolled_bar->up_button,
+	gtk_widget_size_request (vscrolled_bar->up_ebox,
 				 &child_requisition);
 	vscrolled_bar->up_button_width = child_requisition.width;
 	vscrolled_bar->up_button_height = child_requisition.height;
 
-	gtk_widget_size_request (vscrolled_bar->down_button,
+	gtk_widget_size_request (vscrolled_bar->down_ebox,
 				 &child_requisition);
 	vscrolled_bar->down_button_width = child_requisition.width;
 	vscrolled_bar->down_button_height = child_requisition.height;
@@ -334,7 +332,6 @@ e_vscrolled_bar_size_allocate (GtkWidget *widget,
 	GtkAllocation button_allocation, child_allocation;
 	gint border_width;
 
-	g_return_if_fail (widget != NULL);
 	g_return_if_fail (E_IS_VSCROLLED_BAR (widget));
 	g_return_if_fail (allocation != NULL);
 
@@ -345,63 +342,52 @@ e_vscrolled_bar_size_allocate (GtkWidget *widget,
 
 	border_width = GTK_CONTAINER (widget)->border_width;
 
-	child_allocation.x = border_width;
-	child_allocation.y = border_width;
+	child_allocation.x = allocation->x + border_width;
+	child_allocation.y = allocation->y + border_width;
 	child_allocation.width = allocation->width - 2 * border_width;
 	child_allocation.height = allocation->height - 2 * border_width;
 	gtk_widget_size_allocate (bin->child, &child_allocation);
 
-	button_allocation.x = allocation->width - border_width
+	button_allocation.x = allocation->x + allocation->width - border_width
 		- vscrolled_bar->up_button_width
 		- E_VSCROLLED_BAR_BUTTON_X_OFFSET;
-	button_allocation.y = border_width + E_VSCROLLED_BAR_BUTTON_Y_OFFSET;
+	button_allocation.y = allocation->y + border_width + E_VSCROLLED_BAR_BUTTON_Y_OFFSET;
 	button_allocation.width = vscrolled_bar->up_button_width;
 	button_allocation.height = vscrolled_bar->up_button_height;
-	gtk_widget_size_allocate (vscrolled_bar->up_button,
+	gtk_widget_size_allocate (vscrolled_bar->up_ebox,
 				  &button_allocation);
 
-	button_allocation.x = allocation->width - border_width
+	button_allocation.x = allocation->x + allocation->width - border_width
 		- vscrolled_bar->down_button_width
 		- E_VSCROLLED_BAR_BUTTON_X_OFFSET;
-	button_allocation.y = allocation->height - border_width
+	button_allocation.y = allocation->y + allocation->height - border_width
 		- vscrolled_bar->down_button_height
 		- E_VSCROLLED_BAR_BUTTON_Y_OFFSET;
 	button_allocation.width = vscrolled_bar->down_button_width;
 	button_allocation.height = vscrolled_bar->down_button_height;
-	gtk_widget_size_allocate (vscrolled_bar->down_button,
+	gtk_widget_size_allocate (vscrolled_bar->down_ebox,
 				  &button_allocation);
 }
 
-
-static void
-e_vscrolled_bar_draw (GtkWidget    *widget,
-		      GdkRectangle *area)
+static int
+e_vscrolled_bar_expose (GtkWidget      *widget,
+			GdkEventExpose *event)
 {
-	EVScrolledBar *vscrolled_bar;
-	GtkBin *bin;
-	GdkRectangle child_area;
+	EVScrolledBar *vscrolled_bar = E_VSCROLLED_BAR (widget);
+	GtkContainer *container = GTK_CONTAINER (vscrolled_bar);
+	GtkAdjustment *adjustment = vscrolled_bar->adjustment;
 
-	g_return_if_fail (widget != NULL);
-	g_return_if_fail (E_IS_VSCROLLED_BAR (widget));
-	g_return_if_fail (area != NULL);
-  
-	vscrolled_bar = E_VSCROLLED_BAR (widget);
-	bin = GTK_BIN (widget);
+	if (GTK_BIN (vscrolled_bar)->child != NULL)
+		gtk_container_propagate_expose (container, GTK_BIN (vscrolled_bar)->child, event);
 
-	if (bin->child && GTK_WIDGET_VISIBLE (bin->child) &&
-	    gtk_widget_intersect (bin->child, area, &child_area))
-		gtk_widget_draw (bin->child, &child_area);
-  
-	if (GTK_WIDGET_DRAWABLE (vscrolled_bar->up_button) &&
-	    gtk_widget_intersect (vscrolled_bar->up_button, area, &child_area))
-		gtk_widget_draw (vscrolled_bar->up_button, &child_area);
-  
-	if (GTK_WIDGET_DRAWABLE (vscrolled_bar->down_button) &&
-	    gtk_widget_intersect (vscrolled_bar->down_button, area, &child_area))
-		gtk_widget_draw (vscrolled_bar->down_button, &child_area);
+	if (adjustment->value != adjustment->lower)
+		gtk_container_propagate_expose (container, vscrolled_bar->up_ebox, event);
+	if (adjustment->value < adjustment->upper - adjustment->page_size)
+		gtk_container_propagate_expose (container, vscrolled_bar->down_ebox, event);
+
+	return FALSE;
 }
 
-
 static gint
 e_vscrolled_bar_button_press (GtkWidget      *widget,
 			      GdkEventButton *event)
@@ -430,8 +416,8 @@ e_vscrolled_bar_button_press (GtkWidget      *widget,
 
 	if (adjustment->value != new_value) {
 		adjustment->value = new_value;
-		gtk_signal_emit_by_name (GTK_OBJECT (adjustment),
-					 "value_changed");
+		g_signal_emit_by_name (adjustment,
+				       "value_changed");
 		return TRUE;
 	}
 
@@ -446,7 +432,6 @@ e_vscrolled_bar_add (GtkContainer *container,
 	EVScrolledBar *vscrolled_bar;
 	GtkBin *bin;
 
-	g_return_if_fail (container != NULL);
 	g_return_if_fail (E_IS_VSCROLLED_BAR (container));
   
 	vscrolled_bar = E_VSCROLLED_BAR (container);
@@ -476,7 +461,6 @@ static void
 e_vscrolled_bar_remove (GtkContainer *container,
 			GtkWidget    *child)
 {
-	g_return_if_fail (container != NULL);
 	g_return_if_fail (E_IS_VSCROLLED_BAR (container));
 	g_return_if_fail (child != NULL);
 	g_return_if_fail (GTK_BIN (container)->child == child);
@@ -494,7 +478,6 @@ e_vscrolled_bar_forall (GtkContainer *container,
 			GtkCallback   callback,
 			gpointer      callback_data)
 {
-	g_return_if_fail (container != NULL);
 	g_return_if_fail (E_IS_VSCROLLED_BAR (container));
 	g_return_if_fail (callback != NULL);
 
@@ -502,16 +485,6 @@ e_vscrolled_bar_forall (GtkContainer *container,
 						    include_internals,
 						    callback,
 						    callback_data);
-	if (include_internals) {
-		EVScrolledBar *vscrolled_bar;
-
-		vscrolled_bar = E_VSCROLLED_BAR (container);
-      
-		if (vscrolled_bar->up_button)
-			callback (vscrolled_bar->up_button, callback_data);
-		if (vscrolled_bar->down_button)
-			callback (vscrolled_bar->down_button, callback_data);
-	}
 }
 
 
@@ -545,38 +518,39 @@ void
 e_vscrolled_bar_set_adjustment (EVScrolledBar     *vscrolled_bar,
 				GtkAdjustment     *adjustment)
 {
-	g_return_if_fail (vscrolled_bar != NULL);
 	g_return_if_fail (E_IS_VSCROLLED_BAR (vscrolled_bar));
 
 	if (adjustment)
 		g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
 	else
-		adjustment = (GtkAdjustment*) gtk_object_new (GTK_TYPE_ADJUSTMENT, NULL);
+		adjustment = (GtkAdjustment*) g_object_new (GTK_TYPE_ADJUSTMENT, NULL);
 
 	if (vscrolled_bar->adjustment == adjustment)
 		return;
 
 	if (vscrolled_bar->adjustment) {
-		gtk_signal_disconnect_by_func (GTK_OBJECT (vscrolled_bar->adjustment),
-					       GTK_SIGNAL_FUNC (e_vscrolled_bar_adjustment_changed),
-					       vscrolled_bar);
-		gtk_object_unref (GTK_OBJECT (vscrolled_bar->adjustment));
+		g_signal_handlers_disconnect_matched (vscrolled_bar->adjustment,
+						      G_SIGNAL_MATCH_FUNC,
+						      0, 0, 0,
+						      e_vscrolled_bar_adjustment_changed,
+						      NULL);
+		g_object_unref (vscrolled_bar->adjustment);
 	}
 
 	vscrolled_bar->adjustment = adjustment;
-	gtk_object_ref (GTK_OBJECT (vscrolled_bar->adjustment));
+	g_object_ref (vscrolled_bar->adjustment);
 	gtk_object_sink (GTK_OBJECT (vscrolled_bar->adjustment));
 
 	/* I've used connect_after here to avoid a problem when using a
 	   GnomeCanvas as the child widget. When just using connect it would
 	   leave a blank space when one of the buttons is hidden. We want
 	   the GtkLayout to handle the scrolling before we hide any buttons. */
-	gtk_signal_connect_after (GTK_OBJECT (adjustment), "changed",
-				  GTK_SIGNAL_FUNC (e_vscrolled_bar_adjustment_changed),
-				  vscrolled_bar);
-	gtk_signal_connect_after (GTK_OBJECT (adjustment), "value_changed",
-				  GTK_SIGNAL_FUNC (e_vscrolled_bar_adjustment_changed),
-				  vscrolled_bar);
+	g_signal_connect_after (adjustment, "changed",
+				G_CALLBACK (e_vscrolled_bar_adjustment_changed),
+				vscrolled_bar);
+	g_signal_connect_after (adjustment, "value_changed",
+				G_CALLBACK (e_vscrolled_bar_adjustment_changed),
+				vscrolled_bar);
 
 	e_vscrolled_bar_adjustment_changed (adjustment, vscrolled_bar);
 
@@ -601,19 +575,19 @@ e_vscrolled_bar_adjustment_changed (GtkAdjustment *adjustment,
 
 	/* If the adjustment value is not 0, show the up button. */
 	if (adjustment->value != adjustment->lower) {
-		gtk_widget_map (vscrolled_bar->up_button);
-		gdk_window_raise (vscrolled_bar->up_button->window);
+		gtk_widget_map (vscrolled_bar->up_ebox);
+		gdk_window_raise (vscrolled_bar->up_ebox->window);
 	} else {
-		gtk_widget_unmap (vscrolled_bar->up_button);
+		gtk_widget_unmap (vscrolled_bar->up_ebox);
 	}
 
 	/* If the adjustment value is less than the maximum value, show the
 	   down button. */
 	if (adjustment->value < adjustment->upper - adjustment->page_size) {
-		gtk_widget_map (vscrolled_bar->down_button);
-		gdk_window_raise (vscrolled_bar->down_button->window);
+		gtk_widget_map (vscrolled_bar->down_ebox);
+		gdk_window_raise (vscrolled_bar->down_ebox->window);
 	} else {
-		gtk_widget_unmap (vscrolled_bar->down_button);
+		gtk_widget_unmap (vscrolled_bar->down_ebox);
 	}
 }
 
@@ -693,8 +667,8 @@ e_vscrolled_bar_scroll (gpointer data)
 
 	if (adjustment->value != new_value) {
 		adjustment->value = new_value;
-		gtk_signal_emit_by_name (GTK_OBJECT (adjustment),
-					 "value_changed");
+		g_signal_emit_by_name (adjustment,
+				       "value_changed");
 	}
 
 	return retval;
