@@ -25,14 +25,18 @@
  * 02111-1307, USA.
  */
 
-#include "color-group.h"
-#include <glib-object.h>
-#include <gdk/gdkcolor.h>
-#include "gal/util/e-util.h"
+#include <config.h>
+
 #include <string.h>
 
-#define PARENT_TYPE	(G_TYPE_OBJECT)
-#define PARENT_CLASS	gtk_type_class ( gtk_object_get_type ())
+#include <glib-object.h>
+#include <gdk/gdk.h>
+
+#include "gal/util/e-util.h"
+
+#include "color-group.h"
+
+#define PARENT_TYPE	G_TYPE_OBJECT
 
 enum {
         CUSTOM_COLOR_ADD,
