@@ -23,19 +23,22 @@
 
 #include <config.h>
 
+#include <glib-object.h>
+
+#include "gal/util/e-i18n.h"
+#include "gal/util/e-util.h"
+
 #include "e-categories-master-list-dialog-model.h"
 
-#include <gnome.h>
-
-#define PARENT_TYPE e_table_model_get_type()
-ETableModelClass *parent_class;
+#define PARENT_TYPE E_TABLE_MODEL_TYPE
+static ETableModelClass *parent_class;
 
 /*
  * ECategoriesMasterListDialogModel callbacks
  * These are the callbacks that define the behavior of our custom model.
  */
-static void ecmldm_set_arg (GtkObject *o, GtkArg *arg, guint arg_id);
-static void ecmldm_get_arg (GtkObject *object, GtkArg *arg, guint arg_id);
+static void ecmldm_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
+static void ecmldm_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 
 struct ECategoriesMasterListDialogModelPriv {
 	ECategoriesMasterList *ecml;
@@ -45,28 +48,28 @@ struct ECategoriesMasterListDialogModelPriv {
 
 
 enum {
-	ARG_0,
-	ARG_ECML
+	PROP_0,
+	PROP_ECML
 };
 
 #define COLS (3)
 
 static void
-ecmldm_destroy(GtkObject *object)
+ecmldm_dispose(GObject *object)
 {
 	ECategoriesMasterListDialogModel *model = E_CATEGORIES_MASTER_LIST_DIALOG_MODEL(object);
 
 	if (model->priv->ecml) {
 		if (model->priv->ecml_changed_id)
-			gtk_signal_disconnect (GTK_OBJECT (model->priv->ecml),
-					       model->priv->ecml_changed_id);
-		gtk_object_unref (GTK_OBJECT (model->priv->ecml));
+			g_signal_handler_disconnect (model->priv->ecml,
+						     model->priv->ecml_changed_id);
+		g_object_unref (model->priv->ecml);
 	}
 	g_free (model->priv);
 	model->priv = NULL;
 
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+	if (G_OBJECT_CLASS (parent_class)->dispose)
+		(* G_OBJECT_CLASS (parent_class)->dispose) (object);
 }
 
 /* This function returns the number of columns in our ETableModel. */
@@ -175,18 +178,22 @@ ecmldm_value_to_string (ETableModel *etm, int col, const void *value)
 }
 
 static void
-ecmldm_class_init (GtkObjectClass *object_class)
+ecmldm_class_init (GObjectClass *object_class)
 {
 	ETableModelClass *model_class = (ETableModelClass *) object_class;
 
-	parent_class                  = gtk_type_class (PARENT_TYPE);
+	parent_class                  = g_type_class_ref (PARENT_TYPE);
 
-	object_class->destroy         = ecmldm_destroy;
-	object_class->set_arg         = ecmldm_set_arg;
-	object_class->get_arg         = ecmldm_get_arg;
+	object_class->dispose         = ecmldm_dispose;
+	object_class->set_property    = ecmldm_set_property;
+	object_class->get_property    = ecmldm_get_property;
 
-	gtk_object_add_arg_type ("ECategoriesMasterListDialogModel::ecml", E_CATEGORIES_MASTER_LIST_TYPE, 
-				 GTK_ARG_READWRITE, ARG_ECML);
+	g_object_class_install_property (object_class, PROP_ECML,
+					 g_param_spec_object ("ecml",
+							      _( "ECML" ),
+							      _( "ECategoriesMasterListCombo" ),
+							      E_CATEGORIES_MASTER_LIST_TYPE,
+							      G_PARAM_READWRITE));
 
 	model_class->column_count     = ecmldm_col_count;
 	model_class->row_count        = ecmldm_row_count;
@@ -216,26 +223,26 @@ ecml_changed (ECategoriesMasterList *ecml, ECategoriesMasterListDialogModel *ecm
 }
 
 static void
-ecmldm_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
+ecmldm_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
 	ECategoriesMasterListDialogModel *model;
 
-	model = E_CATEGORIES_MASTER_LIST_DIALOG_MODEL (o);
+	model = E_CATEGORIES_MASTER_LIST_DIALOG_MODEL (object);
 	
-	switch (arg_id){
-	case ARG_ECML:
+	switch (prop_id){
+	case PROP_ECML:
 		if (model->priv->ecml) {
 			if (model->priv->ecml_changed_id)
-				gtk_signal_disconnect (GTK_OBJECT (model->priv->ecml),
-						       model->priv->ecml_changed_id);
-			gtk_object_unref (GTK_OBJECT (model->priv->ecml));
+				g_signal_handler_disconnect (model->priv->ecml,
+							     model->priv->ecml_changed_id);
+			g_object_unref (model->priv->ecml);
 		}
-		model->priv->ecml = (ECategoriesMasterList *) GTK_VALUE_OBJECT (*arg);
+		model->priv->ecml = (ECategoriesMasterList *) g_value_get_object (value);
 		if (model->priv->ecml) {
-			gtk_object_ref (GTK_OBJECT (model->priv->ecml));
+			g_object_ref (model->priv->ecml);
 			model->priv->ecml_changed_id =
-				gtk_signal_connect (GTK_OBJECT (model->priv->ecml), "changed",
-						    GTK_SIGNAL_FUNC (ecml_changed), model);
+				g_signal_connect (model->priv->ecml, "changed",
+						  G_CALLBACK (ecml_changed), model);
 		}
 		e_table_model_changed (E_TABLE_MODEL(model));
 		break;
@@ -243,51 +250,35 @@ ecmldm_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 }
 
 static void
-ecmldm_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+ecmldm_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
 	ECategoriesMasterListDialogModel *ecmldm;
 
 	ecmldm = E_CATEGORIES_MASTER_LIST_DIALOG_MODEL (object);
 
-	switch (arg_id) {
-	case ARG_ECML:
-		GTK_VALUE_OBJECT (*arg) = (GtkObject *)ecmldm->priv->ecml;
+	switch (prop_id) {
+	case PROP_ECML:
+		g_value_set_object (value, ecmldm->priv->ecml);
 		break;
 	default:
-		arg->type = GTK_TYPE_INVALID;
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
-GtkType
-e_categories_master_list_dialog_model_get_type (void)
-{
-	static GtkType type = 0;
-
-	if (!type){
-		GtkTypeInfo info = {
-			"ECategoriesMasterListDialogModel",
-			sizeof (ECategoriesMasterListDialogModel),
-			sizeof (ECategoriesMasterListDialogModelClass),
-			(GtkClassInitFunc) ecmldm_class_init,
-			(GtkObjectInitFunc) ecmldm_init,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		type = gtk_type_unique (PARENT_TYPE, &info);
-	}
-
-	return type;
-}
+E_MAKE_TYPE (e_categories_master_list_dialog_model,
+	     "ECategoriesMasterListDialogModel",
+	     ECategoriesMasterListDialogModel,
+	     ecmldm_class_init,
+	     ecmldm_init,
+	     PARENT_TYPE)
 
 ETableModel *
 e_categories_master_list_dialog_model_new (void)
 {
 	ECategoriesMasterListDialogModel *et;
 
-	et = gtk_type_new (e_categories_master_list_dialog_model_get_type ());
+	et = g_object_new (E_CATEGORIES_MASTER_LIST_DIALOG_MODEL_TYPE, NULL);
 	
 	return E_TABLE_MODEL(et);
 }
