@@ -21,15 +21,19 @@
  * 02111-1307, USA.
  */
 
-#include <gtk/gtksignal.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkclist.h>
-#include <gdk/gdkkeysyms.h>
-#include "e-file-selection.h"
-#include "gal/util/e-util.h"
+#include <config.h>
+
 #include <string.h>
 
+#include <gdk/gdkkeysyms.h>
+#include <gtk/gtk.h>
+
+#include "gal/util/e-i18n.h"
+#include "gal/util/e-util.h"
+
+#include "e-file-selection.h"
+
+#define PARENT_TYPE (gtk_file_selection_get_type ())
 static GtkFileSelectionClass *parent_class = NULL;
 
 #if 0
@@ -41,11 +45,9 @@ enum {
 static guint signals [LAST_SIGNAL] = { 0, };
 #endif
 
-#define PARENT_TYPE (gtk_file_selection_get_type ())
-
 enum {
-	ARG_0,
-	ARG_MULTIPLE
+	PROP_0,
+	PROP_MULTIPLE
 };
 
 struct _EFileSelectionPrivate {
@@ -56,29 +58,35 @@ struct _EFileSelectionPrivate {
 };
 
 static void
-efs_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
+efs_get_property (GObject *object,
+		  guint prop_id,
+		  GValue *value,
+		  GParamSpec *pspec)
 {
-	EFileSelection *selection = E_FILE_SELECTION (o);
+	EFileSelection *selection = E_FILE_SELECTION (object);
 
-	switch (arg_id){
-	case ARG_MULTIPLE:
-		GTK_VALUE_BOOL (*arg) = selection->priv->multiple;
+	switch (prop_id){
+	case PROP_MULTIPLE:
+		g_value_set_boolean (value, selection->priv->multiple);
 		break;
 	default:
-		arg->type = GTK_TYPE_INVALID;
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
 static void
-efs_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
+efs_set_property (GObject *object,
+		  guint prop_id,
+		  const GValue *value,
+		  GParamSpec *pspec)
 {
-	EFileSelection *selection = E_FILE_SELECTION (o);
+	EFileSelection *selection = E_FILE_SELECTION (object);
 	GtkWidget *file_list;
 
-	switch (arg_id){
-	case ARG_MULTIPLE:
-		selection->priv->multiple = GTK_VALUE_BOOL (*arg);
+	switch (prop_id){
+	case PROP_MULTIPLE:
+		selection->priv->multiple = g_value_get_boolean (value);
 		file_list = GTK_FILE_SELECTION (selection)->file_list;
 		gtk_clist_set_selection_mode (GTK_CLIST (file_list),
 					      selection->priv->multiple ?
@@ -93,30 +101,32 @@ efs_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 static void
 e_file_selection_class_init (EFileSelectionClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 
-	object_class     = (GtkObjectClass*) klass;
+	object_class     = (GObjectClass*) klass;
 
-	object_class->get_arg = efs_get_arg;
-	object_class->set_arg = efs_set_arg;
+	object_class->get_property = efs_get_property;
+	object_class->set_property = efs_set_property;
 
-	parent_class = gtk_type_class (PARENT_TYPE);
-
-	gtk_object_add_arg_type ("EFileSelection::multiple", GTK_TYPE_BOOL,
-				 GTK_ARG_READWRITE, ARG_MULTIPLE);
+	parent_class = g_type_class_ref (PARENT_TYPE);
 
+	g_object_class_install_property (object_class, PROP_MULTIPLE,
+					 g_param_spec_boolean ("multiple",
+							       _( "Multiple" ),
+							       _( "Multiple" ),
+							       FALSE,
+							       G_PARAM_READWRITE));
 #if 0
 	klass->changed        = NULL;
 
 	signals [CHANGED] =
-		gtk_signal_new ("changed",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (EFileSelectionClass, changed),
-				gtk_marshal_NONE__INT,
-				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
-
-	E_OBJECT_CLASS_ADD_SIGNALS (object_class, signals, LAST_SIGNAL);
+		g_signal_new ("changed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (EFileSelectionClass, changed),
+			      NULL, NULL,
+			      e_marshal_NONE__INT,
+			      G_TYPE_NONE, 1, G_TYPE_INT);
 #endif
 }
 
@@ -134,7 +144,7 @@ e_file_selection_entry_key_press (GtkWidget   *widget,
 	if (event->keyval == GDK_Tab) {
 		int selection_index;
 		efs = E_FILE_SELECTION (user_data);
-		text = gtk_entry_get_text (GTK_ENTRY (efs->priv->selection_entry));
+		text = (char *) gtk_entry_get_text (GTK_ENTRY (efs->priv->selection_entry));
 
 		text = g_strdup (text);
 
@@ -142,13 +152,13 @@ e_file_selection_entry_key_press (GtkWidget   *widget,
 
 		g_free (text);
 
-		text = gtk_entry_get_text (GTK_ENTRY (GTK_FILE_SELECTION (efs)->selection_entry));
+		text = (char *) gtk_entry_get_text (GTK_ENTRY (GTK_FILE_SELECTION (efs)->selection_entry));
 		gtk_entry_set_text (GTK_ENTRY (efs->priv->selection_entry), text);
 
 		selection_index = gtk_editable_get_position (GTK_EDITABLE (GTK_FILE_SELECTION (efs)->selection_entry));
 		gtk_editable_set_position (GTK_EDITABLE (efs->priv->selection_entry), selection_index);
 
-		gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
+		g_signal_stop_emission_by_name (widget, "key_press_event");
 
 		return TRUE;
 	}
@@ -235,10 +245,10 @@ e_file_selection_init (EFileSelection *file_selection)
 	file_selection->priv->in_selection_changed = FALSE;
 	file_selection->priv->selection_entry = entry = gtk_entry_new();
 
-	gtk_signal_connect (GTK_OBJECT (entry), "key_press_event",
-			    GTK_SIGNAL_FUNC (e_file_selection_entry_key_press), file_selection);
-	gtk_signal_connect (GTK_OBJECT (entry), "changed",
-			    GTK_SIGNAL_FUNC (e_file_selection_entry_changed), file_selection);
+	g_signal_connect (entry, "key_press_event",
+			  G_CALLBACK (e_file_selection_entry_key_press), file_selection);
+	g_signal_connect (entry, "changed",
+			  G_CALLBACK (e_file_selection_entry_changed), file_selection);
 	gtk_signal_connect_object (GTK_OBJECT (entry), "focus_in_event",
 				   (GtkSignalFunc) gtk_widget_grab_default,
 				   GTK_OBJECT (GTK_FILE_SELECTION(file_selection)->ok_button));
@@ -258,41 +268,23 @@ e_file_selection_init (EFileSelection *file_selection)
 
 	file_list = GTK_FILE_SELECTION (file_selection)->file_list;
 
-	gtk_signal_connect (GTK_OBJECT (file_list), "select_row",
-			    GTK_SIGNAL_FUNC (row_changed), file_selection);
-	gtk_signal_connect (GTK_OBJECT (file_list), "unselect_row",
-			    GTK_SIGNAL_FUNC (row_changed), file_selection);
+	g_signal_connect (file_list, "select_row",
+			  G_CALLBACK (row_changed), file_selection);
+	g_signal_connect (file_list, "unselect_row",
+			  G_CALLBACK (row_changed), file_selection);
 }
 
-GtkType
-e_file_selection_get_type (void)
-{
-	static GtkType type = 0;
-
-	if (!type)
-	{
-		static const GtkTypeInfo info =
-		{
-			"EFileSelection",
-			sizeof (EFileSelection),
-			sizeof (EFileSelectionClass),
-			(GtkClassInitFunc) e_file_selection_class_init,
-			(GtkObjectInitFunc) e_file_selection_init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL,
-		};
-
-		type = gtk_type_unique (PARENT_TYPE, &info);
-	}
-
-	return type;
-}
+E_MAKE_TYPE (e_file_selection,
+	     "EFileSelection",
+	     EFileSelection,
+	     e_file_selection_class_init,
+	     e_file_selection_init,
+	     PARENT_TYPE)
 
 GtkWidget *
 e_file_selection_new (char *title)
 {
-	EFileSelection *selection = gtk_type_new (e_file_selection_get_type());
+	EFileSelection *selection = g_object_new (E_FILE_SELECTION_TYPE, NULL);
 	gtk_window_set_title (GTK_WINDOW (selection), title);
 	return GTK_WIDGET (selection);
 }
@@ -336,7 +328,7 @@ e_file_selection_get_filenames (EFileSelection *efs)
 		text = strrchr (filesel_path, '/');
 		if (text) text[1] = '\0';   /* one past the / */
 
-		text = gtk_entry_get_text (GTK_ENTRY (efs->priv->selection_entry));
+		text = (char *) gtk_entry_get_text (GTK_ENTRY (efs->priv->selection_entry));
 
 		ret_val = g_new (char *, 2);
 		ret_val[0] = (filesel_path && *text != '/') ? g_strconcat (filesel_path, text, NULL) : g_strdup (text);
