@@ -30,15 +30,15 @@
 
 #include <config.h>
 
-#include <gtk/gtkentry.h>
-#include <gtk/gtksignal.h>
+#include <gtk/gtk.h>
 #include <libgnomecanvas/gnome-canvas.h>
 #include <libgnomecanvas/gnome-canvas-pixbuf.h>
 #include <libgnomecanvas/gnome-canvas-rect-ellipse.h>
-#include <gal/util/e-util.h>
+
+#include "gal/util/e-util.h"
+
 #include "e-colors.h"
 #include "widget-color-combo.h"
-#include "gal/util/e-util.h"
 
 enum {
 	CHANGED,
